@@ -1,4 +1,4 @@
-# -*- coding: future_annotations -*- 
+# -*- coding: utf-8 -*-
 import six
 import sys
 
@@ -8,10 +8,11 @@ class open_file:
     Context manager that opens a filename and closes it on exit, but does
     nothing for file-like objects.
     """
+
     def __init__(self, filename, *args, **kwargs) -> None:
-        self.closing = kwargs.pop('closing', False)
+        self.closing = kwargs.pop("closing", False)
         if filename is None:
-            stream = sys.stdout if 'w' in args else sys.stdin
+            stream = sys.stdout if "w" in args else sys.stdin
             if six.PY3:
                 self.fh = open(stream.fileno(), *args, **kwargs)
             else:
